package com.ftdi.io;

import java.util.concurrent.TimeoutException;

public abstract class SerialPort {	

	public enum DATABITS {
		DATABITS_8,
		DATABITS_7,
		DATABITS_6,
		DATABITS_5
	};

	public enum STOPBITS {
		STOPBITS_1,
		STOPBITS_1_5,
		STOPBITS_2
	};

	public enum PARITY {
		PARITY_NONE,
		PARITY_ODD,
		PARITY_EVEN,
		PARITY_MARK,
		PARITY_SPACE
	};
	
	/** Open a serial port
	 * 
	 * @param portname The name of the port to open
	 * @param baud The baud rate
	 * @param databits The number of data bits
	 * @param stopbits The number of stop bits
	 * @param parity The style of parity
	 */
	public abstract void open(String portname, int baud, DATABITS databits, STOPBITS stopbits, PARITY parity);
	
	/** Close a serial port
	 * 
	 */
	public abstract void close();
	
	/** Read a byte from a serial port
	 * 
	 * @return The byte read or 0;
	 */
	public abstract byte read();
	
	/** Read a byte from a serial port within a time limit
	 * 
	 * @param timeout The timeout in milliseconds to wait
	 * @return The byte read or 0
	 * @throws TimeoutException
	 */
	public abstract byte read(int timeout) throws TimeoutException;
	
	/** Write a byte to a serial port
	 * 
	 * @param b The byte to write
	 */
	public abstract void write(byte b);
	
	/** Write a String to a serial port
	 * 
	 * @param s The string to write
	 */
	public abstract void write(String s);
	
	/** Get the number of available bytes in the input buffer
	 * 
	 * @return The number of bytes available
	 */
	public abstract int available();
	
	/** Empty the input and output buffers
	 * 
	 */
	public abstract void clear();
	
	/** Wait for all data to be transmitted
	 * 
	 */
	public abstract void flush();
	
	/** Add a receive listener
	 * 
	 * @param spl The receive listener
	 */
	public abstract void addReceiveListener(SerialPortListener spl);
	
	/** Add a transmit listener
	 * 
	 * @param spl The transmit listener
	 */
	public abstract void addTransmitListener(SerialPortListener spl);
}
